// Base Dependencies
import { Component, ViewChild } from '@angular/core';
import { NavController, NavParams, MenuController, normalizeURL } from 'ionic-angular';
import uuid from 'uuid/v1';
// Ionic Native plugins
import { StatusBar } from '@ionic-native/status-bar';
import { CameraPreview } from '@ionic-native/camera-preview';
import { File } from '@ionic-native/file';
import { DeviceMotion } from '@ionic-native/device-motion';
// Components
import { EditComponent } from '../edit/edit';
// Helpers
import { base64toBlob, rotateBase64Image } from '../../helpers/picture-mutations';
var CameraComponent = /** @class */ (function () {
    function CameraComponent(navCtrl, navParams, menu, statusBar, cameraPreview, file, deviceMotion) {
        this.navCtrl = navCtrl;
        this.navParams = navParams;
        this.menu = menu;
        this.statusBar = statusBar;
        this.cameraPreview = cameraPreview;
        this.file = file;
        this.deviceMotion = deviceMotion;
        this.pictures = [];
        this.doesExit = false;
        // Flash Mode
        this.showFlashOptions = false;
        this._activeFlashMode = 'auto';
        this.callback = this.navParams.get('callback');
        this.pictureOptions = this.navParams.get('pictureOptions') || {
            quality: 80,
            width: 4096,
            height: 4096
        };
        this.translations = this.navParams.get('translations') || {
            cancel: 'Cancel',
            finish: 'Finish',
            auto: 'AUTO',
            on: 'On',
            off: 'Off'
        };
    }
    Object.defineProperty(CameraComponent.prototype, "activeFlashMode", {
        get: function () {
            return this._activeFlashMode;
        },
        set: function (newValue) {
            var _this = this;
            this.cameraPreview.setFlashMode(newValue)
                .then(function () {
                _this._activeFlashMode = newValue;
                _this.showFlashOptions = false;
            })
                .catch(function (err) {
                _this.errorHandler(err);
            });
        },
        enumerable: true,
        configurable: true
    });
    CameraComponent.prototype.ionViewWillEnter = function () {
        this.menu.swipeEnable(false);
        this.statusBar.styleBlackOpaque();
    };
    CameraComponent.prototype.ionViewDidEnter = function () {
        var _this = this;
        this.startCam();
        this.motionSubscription = this.deviceMotion.watchAcceleration({ frequency: 500 }).subscribe(function (acceleration) {
            _this.deviceOrientation = acceleration;
        });
    };
    CameraComponent.prototype.ionViewWillLeave = function () {
        if (this.doesExit) {
            this.menu.swipeEnable(true);
            this.statusBar.styleDefault();
        }
    };
    CameraComponent.prototype.ionViewDidLeave = function () {
        if (this.doesExit) {
            this.stopCam();
            this.motionSubscription.unsubscribe();
        }
    };
    CameraComponent.prototype.focus = function (event) {
        var headerHeight = this.header.nativeElement.offsetHeight;
        var footerHeight = this.footer.nativeElement.offsetHeight;
        var xPoint = event.offsetX;
        var yPoint = (event.offsetY - headerHeight) / (window.screen.height - headerHeight - footerHeight) * window.screen.height;
        this.cameraPreview.tapToFocus(xPoint, yPoint)
            .then(function () {
        })
            .catch(function (err) {
            console.error(err);
        });
    };
    CameraComponent.prototype.switchCamera = function () {
        var _this = this;
        this.cameraPreview.switchCamera()
            .then(function () {
        })
            .catch(function (err) {
            _this.errorHandler(err);
        });
    };
    CameraComponent.prototype.cancel = function () {
        var _this = this;
        var i = this.pictures.length;
        this.pictures.forEach(function (picture) {
            picture.fileEntry.remove(function () {
                i--;
                if (i === 0) {
                    _this.pictures = [];
                    _this.exit();
                }
            }, function (err) {
                _this.errorHandler(err);
            });
        });
        if (i === 0) {
            this.exit();
        }
    };
    CameraComponent.prototype.takePicture = function () {
        var _this = this;
        var imageOrientation = this.getImageOrientation();
        this.cameraPreview.takePicture(this.pictureOptions)
            .then(function (picture) {
            var fileOptions = {
                replace: true
            };
            return _this.file.writeFile(_this.file.cacheDirectory, uuid() + '.jpeg', base64toBlob(picture, 'image/jpeg'), fileOptions)
                .then(function (fileEntry) {
                var normalizedURL = normalizeURL(fileEntry.toURL());
                _this.pictures.push({
                    fileEntry: fileEntry,
                    normalizedURL: normalizedURL,
                    base64Data: picture,
                    imageOrientation: imageOrientation
                });
            })
                .catch(function (err) {
                _this.errorHandler(err);
            });
        })
            .catch(function (err) {
            _this.errorHandler(err);
        });
    };
    CameraComponent.prototype.finish = function () {
        var _this = this;
        var promises = [];
        this.pictures.forEach(function (picture) {
            promises.push(new Promise(function (resolve, reject) {
                rotateBase64Image(picture.base64Data, picture.imageOrientation)
                    .then(function (imageData) {
                    picture.imageOrientation = 0;
                    picture.base64Data = imageData;
                    picture.fileEntry.createWriter(function (fileWriter) {
                        fileWriter.onwriteend = function () {
                            resolve(picture);
                        };
                        fileWriter.onerror = function (e) {
                            reject(e);
                        };
                        var file = base64toBlob(imageData, 'image/jpeg');
                        fileWriter.truncate(file.size);
                        fileWriter.write(file);
                    }, function (err) {
                        reject(err);
                    });
                })
                    .catch(function (err) {
                    reject(err);
                });
            }));
        });
        Promise.all(promises)
            .then(function (results) {
            _this.callback({
                pictures: results
            });
            _this.doesExit = true;
            _this.navCtrl.pop();
        })
            .catch(function (err) {
            _this.errorHandler(err);
        });
    };
    CameraComponent.prototype.editPicture = function (picture, index) {
        this.navCtrl.push(EditComponent, {
            picture: picture,
            array: this.pictures,
            index: index
        });
    };
    CameraComponent.prototype.errorHandler = function (error) {
        this.callback({
            error: error
        });
        this.doesExit = true;
        this.navCtrl.pop();
    };
    CameraComponent.prototype.exit = function () {
        this.callback({
            pictures: this.pictures
        });
        this.doesExit = true;
        this.navCtrl.pop();
    };
    CameraComponent.prototype.startCam = function () {
        var _this = this;
        var headerHeight = this.header.nativeElement.offsetHeight;
        var footerHeight = this.footer.nativeElement.offsetHeight;
        var cameraPreviewOpts = {
            x: 0,
            y: headerHeight,
            width: window.screen.width,
            height: window.screen.height - headerHeight - footerHeight,
            camera: 'rear',
            tapPhoto: false,
            previewDrag: false,
            toBack: true,
            alpha: 1
        };
        this.cameraPreview.startCamera(cameraPreviewOpts)
            .then(function () {
            _this.cameraPreview.setFlashMode('auto')
                .then(function () {
            })
                .catch(function (err) {
                _this.errorHandler(err);
            });
        })
            .catch(function (err) {
            if (err.indexOf('Camera already started') !== -1) {
                _this.cameraPreview.setFlashMode('auto')
                    .then(function () {
                })
                    .catch(function (err) {
                    _this.errorHandler(err);
                });
            }
            else {
                _this.errorHandler(err);
            }
        });
    };
    CameraComponent.prototype.stopCam = function () {
        this.cameraPreview.stopCamera();
    };
    CameraComponent.prototype.getImageOrientation = function () {
        if (this.deviceOrientation) {
            // If landscape
            if (Math.abs(this.deviceOrientation.x) > Math.abs(this.deviceOrientation.y)) {
                if (this.deviceOrientation.x > 0) {
                    return 270;
                }
                else {
                    return 90;
                }
            }
            else {
                // Portrait upside-down
                if (this.deviceOrientation.y < 0) {
                    return 180;
                }
                else {
                    // Right-side up
                    return 0;
                }
            }
        }
        else {
            return 0;
        }
    };
    CameraComponent.decorators = [
        { type: Component, args: [{
                    selector: 'ionic-multi-camera',
                    template: '<ion-header no-border #header><ion-toolbar color="camera"><ion-buttons left><button ion-button icon-only clear (click)="showFlashOptions = !showFlashOptions"><ion-icon name="flash"></ion-icon></button></ion-buttons><ion-segment color="light" *ngIf="showFlashOptions" [(ngModel)]="activeFlashMode"><ion-segment-button value="auto">{{ translations.auto }}</ion-segment-button><ion-segment-button value="on">{{ translations.on }}</ion-segment-button><ion-segment-button value="off">{{ translations.off }}</ion-segment-button></ion-segment><ion-buttons right><button ion-button icon-only clear *ngIf="!showFlashOptions" (click)="switchCamera()"><ion-icon name="reverse-camera"></ion-icon></button></ion-buttons></ion-toolbar></ion-header><ion-content (click)="focus($event)"></ion-content><ion-footer no-border #footer><ion-row nowrap justify-content-start align-items-center><ion-col *ngFor="let picture of pictures; let i = index" col-auto><img tappable [src]="\'data:image/jpeg;base64,\' + picture.base64Data" [style.transform]="\'rotate(\' + picture.imageOrientation + \'deg)\'" class="thumbnail" (click)="editPicture(picture, i)"></ion-col></ion-row><ion-toolbar color="camera" class="footer-bar"><ion-buttons left><button ion-button (click)="cancel()">{{ translations.cancel }}</button></ion-buttons><button ion-button icon-only clear large color="light" (click)="takePicture()" class="photo-button"><ion-icon name="ios-radio-button-on-outline"></ion-icon></button><ion-buttons right><button ion-button (click)="finish()">{{ translations.finish }}</button></ion-buttons></ion-toolbar></ion-footer>',
                    styles: [".photo-button {  font-size: 50px;  position: absolute;  left: 50%;  transform: translate(-50%, -50%); }.footer-bar {  padding-bottom: 30px;  padding-bottom: calc(30px + constant(safe-area-inset-bottom)) !important;  padding-bottom: calc(30px + env(safe-area-inset-bottom)) !important;  min-height: 110px;  min-height: calc(110px + constant(safe-area-inset-bottom)) !important;  min-height: calc(110px + env(safe-area-inset-bottom)) !important; }ion-row {  overflow-x: scroll; }.thumbnail {  width: 60px;  height: 60px;  min-width: 60px;  min-height: 60px; }"]
                },] },
    ];
    /** @nocollapse */
    CameraComponent.ctorParameters = function () { return [
        { type: NavController, },
        { type: NavParams, },
        { type: MenuController, },
        { type: StatusBar, },
        { type: CameraPreview, },
        { type: File, },
        { type: DeviceMotion, },
    ]; };
    CameraComponent.propDecorators = {
        'header': [{ type: ViewChild, args: ['header',] },],
        'footer': [{ type: ViewChild, args: ['footer',] },],
    };
    return CameraComponent;
}());
export { CameraComponent };
//# sourceMappingURL=camera.js.map